/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.model.minfo;

import org.apache.log4j.Logger;

import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.access.RiverAccess;
import org.dive4elements.river.artifacts.context.RiverContext;
import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.model.DataFacet;
import org.dive4elements.river.artifacts.math.MovingAverage;
import org.dive4elements.river.artifacts.model.ZoomScale;
import org.dive4elements.river.artifacts.states.DefaultState.ComputeType;


public class BedDiffEpochFilterFacet
extends DataFacet
{
    private static Logger logger = Logger.getLogger(BedDiffEpochFilterFacet.class);

    public BedDiffEpochFilterFacet() {
    }

    public BedDiffEpochFilterFacet(int idx, String name, String description,
        ComputeType type, String stateId, String hash) {
        super(idx, name, description, type, hash, stateId);
        this.metaData.put("X", "chart.longitudinal.section.xaxis.label");
        this.metaData.put("Y", "chart.beddifference.height.yaxis.label");
    }

    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("Get data for bed density at index: " + index);

        D4EArtifact flys = (D4EArtifact) artifact;

        CalculationResult res = (CalculationResult) flys.compute(context, hash,
            stateId, type, false);

        BedDiffEpochResult[] data =
            (BedDiffEpochResult[]) res.getData(); // TODO CAST TO SPECIFIC CLASS
        Double start = (Double)context.getContextValue("startkm");
        Double end = (Double)context.getContextValue("endkm");

        if(start != null && end != null) {
            RiverContext fc = (RiverContext)context.globalContext();
            ZoomScale scales = (ZoomScale)fc.get("zoomscale");
            RiverAccess access = new RiverAccess((D4EArtifact)artifact);
            String river = access.getRiverName();

            double radius = scales.getRadius(river, start, end);
            BedDiffEpochResult oldData = data[index];
            BedDiffEpochResult newData = new BedDiffEpochResult();
            newData.setStart(oldData.getStart());
            newData.setEnd(oldData.getEnd());
            double[][] diffs = MovingAverage.weighted(oldData.getDifferencesData(), radius);
            double[][] heights1 = MovingAverage.weighted(oldData.getHeights1Data(), radius);
            double[][] heights2 = MovingAverage.weighted(oldData.getHeights2Data(), radius);
            for(int j = 0; j < diffs[0].length; j++) {
                newData.addKm(diffs[0][j]);
                newData.addDifference(diffs[1][j]);
                newData.addHeight1(heights1[1][j]);
                newData.addHeight2(heights2[1][j]);
            }
            return newData;
        }
        return data != null && data.length > index ? data[index] : null;
    }

    /** Copy deeply. */
    @Override
    public Facet deepCopy() {
        BedDiffEpochFilterFacet copy = new BedDiffEpochFilterFacet();
        copy.set(this);
        copy.type = type;
        copy.hash = hash;
        copy.stateId = stateId;
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
